-- modified from dawn's AreaEater

---@param user Entity
---@param props CardProperties
---@param tile Tile
local function attack_tile(user, props, tile)
    local hitbox = Hitbox.new(user:team())
    hitbox:set_hit_props(HitProps.new(
        props.damage,
        props.hit_flags,
        props.element,
        user:context(),
        Drag.None
    ))
    user:field():spawn(hitbox, tile)
end

---@param user Entity
---@param props CardProperties
---@param tile Tile?
local function eat_tile(user, props, tile)
    if not tile then
        return
    end

    if tile:is_reserved() then
        attack_tile(user, props, tile)
        return
    end

    local previous_tile = tile
    local current_tile = tile:get_tile(user:facing(), 1)

    while current_tile do
        -- move tile state
        local current_tile_state

        if current_tile:can_set_state(TileState.Normal) then
            current_tile_state = current_tile:state()
        else
            current_tile_state = TileState.EatenArea
        end

        previous_tile:set_state(current_tile_state)
        previous_tile:set_team(current_tile:team())

        -- move entities
        current_tile:find_entities(function(ent)
            if Character.from(ent) ~= nil or Obstacle.from(ent) ~= nil then
                -- todo engine has an issue if this is reversed?
                current_tile:remove_entity_by_id(ent:id())
                previous_tile:add_entity(ent)
            end

            return false
        end)

        previous_tile = current_tile

        if current_tile_state == TileState.EatenArea then
            -- end
            current_tile = nil
        else
            -- advance
            current_tile = current_tile:get_tile(user:facing(), 1)
        end
    end
end

---@param user Entity
---@param props CardProperties
function card_init(user, props)
    local action = Action.new(user, "PLAYER_IDLE")
    action:set_lockout(ActionLockout.new_sequence())

    local sfx = Resources.load_audio("012D_0000.wav")

    action.on_execute_func = function()
        local field = user:field()

        local step1 = action:create_step()

        local fx = Artifact.new()
        fx:set_facing(user:facing())
        fx:set_texture(Resources.load_texture("WarRock_exeOSS_battle.png"))

        local anim = fx:animation()
        anim:load("area_eater.animation")
        anim:set_state("APPEAR")
        anim:apply(fx:sprite())
        anim:on_complete(function()
            Resources.play_audio(sfx)

            anim:set_state("SLAP")
            anim:on_frame(4, function()
                local forward_tile = user:get_tile(user:facing(), 1)

                if forward_tile then
                    eat_tile(user, props, forward_tile)
                    eat_tile(user, props, forward_tile:get_tile(Direction.Up, 1))
                    eat_tile(user, props, forward_tile:get_tile(Direction.Down, 1))
                end
            end)

            anim:on_complete(function()
                fx:erase()
                user:reveal()
                step1:complete_step()
            end)
        end)

        field:spawn(fx, user:current_tile())
        user:hide()
    end

    return action
end
